# flake8: noqa
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('quizblock', '__first__'),
        ('pagetree', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='QuestionUserLock',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('question_used', models.NullBooleanField()),
                ('question_current', models.NullBooleanField()),
                ('question', models.ForeignKey(to='quizblock.Question')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='QuizRandom',
            fields=[
                ('quiz_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='quizblock.Quiz')),
                ('quiz_name', models.CharField(max_length=50)),
                ('quiz_type', models.TextField(blank=True)),
            ],
            options={
            },
            bases=('quizblock.quiz',),
        ),
        migrations.AddField(
            model_name='questionuserlock',
            name='quiz',
            field=models.ForeignKey(to='quizblock_random.QuizRandom'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='questionuserlock',
            name='section',
            field=models.ForeignKey(to='pagetree.Section'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='questionuserlock',
            name='user',
            field=models.ForeignKey(to=settings.AUTH_USER_MODEL),
            preserve_default=True,
        ),
    ]



from django import template
register = template.Library()


@register.assignment_tag
def get_random_question(section, quiz_random, user):
    return quiz_random.get_random_question(section, quiz_random, user)



from django import forms
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.db import models
from pagetree.models import Section
from quizblock.models import Quiz, Question, Submission
from random import randint


class QuizRandom(Quiz):
    display_name = "Quiz Random"
    template_file = "quizblock_random/quizblock_random.html"
    quiz_name = models.CharField(max_length=50)
    quiz_type = models.TextField(blank=True)

    def get_random_question(self, section, quiz, user):
        question_set = self.question_set.all()
        set_len = len(question_set)
        if set_len > 0:
            rand_int = randint(0, set_len-1)
            for question in question_set:
                try:
                    current_question = QuestionUserLock.objects.filter(
                        quiz_id=question.quiz.id
                    ).get(
                        question_current=True,
                        user_id=user.id)
                    qid = current_question.question_id
                    question = Question.objects.get(id=qid)
                    return question

                except QuestionUserLock.DoesNotExist:
                    question = question_set[rand_int]
                    self.set_question_userlock(section, question, user)
                    return question
        else:
            return

    def set_question_userlock(self, section, question, user):
        qul = QuestionUserLock.create(section, question, user)
        qul.question_current = True
        qul.question_used = False
        qul.save()

    def get_random_question_set(self, section, block, question, user):
        question_set = QuestionUserLock.objects.filter(section_id=section.id)
        return question_set

    def pageblock(self):
        return self.pageblocks.all().first()

    def quiz(self):
        return self.quiz.all()[0]

    def clear_user_submissions(self, user):
        self.unset_question_userlock(user)
        Submission.objects.filter(user=user, quiz=self).delete()

    def unset_question_userlock(self, user):
        QuestionUserLock.objects.filter(user=user, quiz=self).delete()

    def edit(self, vals, files):
        self.quiz_type = vals.get('quiz_type', '')
        self.description = vals.get('description', '')
        self.rhetorical = vals.get('rhetorical', '')
        self.allow_redo = vals.get('allow_redo', '')
        self.show_submit_state = vals.get('show_submit_state', False)
        self.save()

    def edit_form(self):
        class EditForm(forms.Form):
            quiz_type = forms.CharField(widget=forms.widgets.Textarea(),
                                        initial=self.quiz_type)
            description = forms.CharField(widget=forms.widgets.Textarea(),
                                          initial=self.description)
            rhetorical = forms.BooleanField(initial=self.rhetorical)
            allow_redo = forms.BooleanField(initial=self.allow_redo)
            show_submit_state = forms.BooleanField(
                initial=self.show_submit_state)
            alt_text = (
                "<a href=\"" + reverse("edit-quiz-random", args=[self.id])
                + "\">manage questions/answers</a>")
        return EditForm()

    def get_question(self):
        return

    @classmethod
    def add_form(self):
        class AddForm(forms.Form):
            quiz_type = forms.CharField(widget=forms.widgets.Textarea())
            description = forms.CharField(widget=forms.widgets.Textarea())
            rhetorical = forms.BooleanField()
            allow_redo = forms.BooleanField()
            show_submit_state = forms.BooleanField(initial=True)
        return AddForm()

    @classmethod
    def create(self, request):
        return QuizRandom.objects.create(
            quiz_type=request.POST.get('quiz_type', ''),
            description=request.POST.get('description', ''),
            rhetorical=request.POST.get('rhetorical', ''),
            allow_redo=request.POST.get('allow_redo', ''),
            show_submit_state=request.POST.get('show_submit_state', False))


class QuestionUserLock(models.Model):
    section = models.ForeignKey(Section,)
    quiz = models.ForeignKey(QuizRandom)
    question = models.ForeignKey(Question)
    user = models.ForeignKey(User)
    question_used = models.NullBooleanField(null=True)
    question_current = models.NullBooleanField(null=True)

    def set_question_user_lock(self, question, user):
        try:
            current_question = QuestionUserLock.objects.filter(
                quiz_id=question.quiz.id).get(question_current=True)
            qid = current_question.question_id
            question = Question.objects.get(id=qid)
            return question
        except QuestionUserLock.DoesNotExist:
            return None

    @classmethod
    def create(self, section, question, user):
        return QuestionUserLock.objects.create(
            section_id=section.id,
            quiz_id=question.quiz.id,
            question_id=question.id,
            user_id=user.id
            )



from django.test import TestCase
from quizblock_random.models import QuizRandom, Submission, QuestionUserLock
from django.contrib.auth.models import User
from .factories import (
    Dummy, QuestionUserLockFactory, CustomPagetreeModuleFactory,
    QuestionFactory, UserFactory, QuizRandomFactory)


class FakeReq(object):
    def __init__(self):
        self.POST = dict()


class TestBasics(TestCase):
    def test_create(self):
        q = QuizRandom()
        self.assertNotEqual(q, None)

    def test_add_form(self):
        f = QuizRandom.add_form()
        self.assertTrue('allow_redo' in f.fields)
        self.assertTrue('show_submit_state' in f.fields)

    def test_create_method(self):
        r = FakeReq()
        q = QuizRandom.create(r)
        self.assertEquals(q.description, '')
        self.assertEquals(q.display_name, 'Quiz Random')
        self.assertFalse(q.show_submit_state)
        self.assertFalse(q.allow_redo)

    def test_create_method_two(self):
        r = FakeReq()
        r.POST['show_submit_state'] = 'on'
        r.POST['allow_redo'] = 'on'
        q = QuizRandom.create(r)
        self.assertEquals(q.description, '')
        self.assertEquals(q.display_name, 'Quiz Random')
        self.assertTrue(q.show_submit_state)
        self.assertTrue(q.allow_redo)

    def test_dict_roundtrip(self):
        q1 = QuizRandom(description="first", show_submit_state=False)
        d = q1.as_dict()
        q2 = QuizRandom(description="second")
        q2.import_from_dict(d)
        self.assertEquals(q2.description, "first")
        self.assertEquals(q1.allow_redo, q2.allow_redo)
        self.assertEquals(q1.show_submit_state, q2.show_submit_state)

    def test_create_from_dict(self):
        q = QuizRandom(description="first")
        d = q.as_dict()
        q2 = QuizRandom.create_from_dict(d)
        self.assertEquals(q2.description, "first")
        self.assertEquals(q.allow_redo, q2.allow_redo)
        self.assertEquals(q.show_submit_state, q2.show_submit_state)

    def test_create_from_dict_defaults(self):
        d = {
            'description': 'Test QuizRandom',
            'rhetorical': True,
            'questions': [],
        }
        q = QuizRandom.create_from_dict(d)
        self.assertEquals(q.description, 'Test QuizRandom')
        self.assertEquals(q.allow_redo, True)
        self.assertEquals(q.show_submit_state, True)

    def test_import_from_dict_defaults(self):
        d = {
            'description': 'Test QuizRandom',
            'rhetorical': True,
            'questions': [],
        }
        q = QuizRandom()
        q.import_from_dict(d)
        self.assertEqual(q.description, 'Test QuizRandom')
        self.assertEqual(q.allow_redo, True)
        self.assertEqual(q.show_submit_state, True)
        self.assertEqual(q.submission_set.count(), 0)
        self.assertEqual(q.question_set.count(), 0)

    def test_import_from_dict_defaults_2(self):
        d = {
            'description': 'Test QuizRandom',
            'allow_redo': False,
            'rhetorical': True,
            'show_submit_state': True,
            'questions': [{
                'text': 'Test Question',
                'question_type': 'single choice',
                'explanation': '',
                'intro_text': '',
                'answers': [],
            }],
        }
        q = QuizRandom()
        q.import_from_dict(d)
        self.assertEqual(q.description, 'Test QuizRandom')
        self.assertEqual(q.allow_redo, False)
        self.assertEqual(q.show_submit_state, True)
        self.assertEqual(q.submission_set.count(), 0)
        self.assertEqual(q.question_set.count(), 1)

    def test_edit(self):
        q = QuizRandom()
        q.edit(dict(description='foo',
                    allow_redo='0', show_submit_state='on'), None)
        self.assertEqual(q.description, 'foo')
        self.assertEqual(q.allow_redo, '0')
        self.assertTrue(q.show_submit_state)

    def test_edit_two(self):
        q = QuizRandom()
        q.edit(dict(description='foo'), None)
        self.assertEqual(q.description, 'foo')
        self.assertFalse(q.allow_redo)
        self.assertFalse(q.show_submit_state)


class UserTests(TestCase):
    def setUp(self):
        self.u = User.objects.create(username="testuser")

    def test_submit(self):
        q = QuizRandom.objects.create()
        self.assertFalse(q.unlocked(self.u))
        q.submit(self.u, dict(foo='bar'))
        self.assertTrue(q.unlocked(self.u))
        q.clear_user_submissions(self.u)
        self.assertFalse(q.unlocked(self.u))


class TestIsUserCorrect(TestCase):

    def setUp(self):
        self.user = User.objects.create(username="testuser")
        self.quiz = QuizRandom.objects.create()

    def test_no_questions(self):
        self.assertEquals(self.quiz.score(self.user), None)


class SubmissionTest(TestCase):
    def test_unicode(self):
        quiz = QuizRandom.objects.create()
        user = User.objects.create(username="testuser")
        s = Submission.objects.create(quiz=quiz, user=user)
        self.assertTrue(
            str(s).startswith("quiz %d submission by testuser" % quiz.id))


class TestQuestionUserLock(TestCase):
    def test_create(self):
        s = Dummy()
        q = Dummy()
        q.quiz = Dummy()
        u = Dummy()
        qul = QuestionUserLock.create(s, q, u)
        self.assertIsNotNone(qul)

    def test_set_question_user_lock(self):
        s = CustomPagetreeModuleFactory()
        qul = QuestionUserLockFactory(section=s.root)
        self.assertIsNone(
            qul.set_question_user_lock(qul.question, None))
        qul.quiz = qul.question.quiz
        qul.save()
        self.assertIsNotNone(
            qul.set_question_user_lock(qul.question, None))

    def test_quiz_set_question_userlock(self):
        s = CustomPagetreeModuleFactory().root
        question = QuestionFactory()
        q = question.quiz
        u = UserFactory()
        q.set_question_userlock(s, question, u)

        r = q.get_random_question_set(s, None, None, None)
        self.assertTrue(len(r) > 0)

        q.unset_question_userlock(u)
        r = q.get_random_question_set(s, None, None, None)
        self.assertEqual(len(r), 0)

    def test_get_random_question_empty(self):
        q = QuizRandomFactory()
        self.assertIsNone(q.get_random_question(None, None, None))

    def test_get_random_question_no_userlock(self):
        s = CustomPagetreeModuleFactory().root
        question = QuestionFactory()
        q = question.quiz
        u = UserFactory()
        r = q.get_random_question(s, q, u)
        self.assertEqual(r, question)

    def test_get_random_question_with_userlock(self):
        s = CustomPagetreeModuleFactory().root
        question = QuestionFactory()
        q = question.quiz
        u = UserFactory()
        q.set_question_userlock(s, question, u)
        r = q.get_random_question(s, q, u)
        self.assertEqual(r, question)



import factory
from quizblock_random.models import Question, QuestionUserLock, QuizRandom
from pagetree.tests.factories import UserFactory
from pagetree.tests.factories import HierarchyFactory


class CustomPagetreeModuleFactory(object):
    def __init__(self):
        hierarchy = HierarchyFactory(name='main', base_url='/pages/')
        root = hierarchy.get_root()
        root.add_child_section_from_dict({
            'label': 'Welcome to the Intro Page',
            'slug': 'intro',
            'children': [
                {
                    'label': 'Step 1',
                    'slug': 'step-1',
                    'pageblocks': [{
                        'block_type': 'Text Block'
                    }]
                },
                {
                    'label': 'Step 2',
                    'slug': 'step-2',
                    'pageblocks': [{
                        'block_type': 'My Block Name'
                    }]
                },
            ]
        })

        self.root = root


class Dummy(object):
    id = 1


class QuizRandomFactory(factory.DjangoModelFactory):
    class Meta:
        model = QuizRandom

    quiz_name = "Random Quiz"
    quiz_type = "Random Quiz Type"


class QuestionFactory(factory.DjangoModelFactory):
    class Meta:
        model = Question
    quiz = factory.SubFactory(QuizRandomFactory)


class QuestionUserLockFactory(factory.DjangoModelFactory):
    class Meta:
        model = QuestionUserLock

    quiz = factory.SubFactory(QuizRandomFactory)
    user = factory.SubFactory(UserFactory)
    question_used = True
    question_current = True
    question = factory.SubFactory(QuestionFactory)



from django import template
register = template.Library()


@register.assignment_tag
def get_questions(section, block, question, user):
    block.get_random_question_set(section, block, question, user)
    is_correct = question.is_user_correct(user)
    return is_correct
